/* 
 * The contents of this file is licenced. You may obtain a copy of
 * the license at http://sieve.mozdev.org or request it via email 
 * from the author. Do not remove or change this comment. 
 * 
 * The initial author of the code is:
 *   Thomas Schmid <schmid-thomas@gmx.net>
 */
 
"use strict";
 
 
//   Usage:  ":lower" / ":upper" / ":lowerfirst" / ":upperfirst" /
//           ":quotewildcard" / ":length"
 
// set [MODIFIER] <name: string> <value: string>


function SieveSetVariable(docshell,id) 
{
  SieveAbstractElement.call(this,docshell,id); 
  
  this.whiteSpace = this._createByName("whitespace"," ");
  this.flaglist = this._createByName("stringlist");
  this.semicolon = this._createByName("atom/semicolon");
}

SieveSetVariable.prototype = Object.create(SieveAbstractElement.prototype);
SieveSetVariable.prototype.constructor = SieveSetVariable;

SieveSetVariable.isElement
    = function (parser, lexer)
{
  return parser.startsWith("set");
}
    
SieveSetVariable.isCapable = function (capabilities) {
  return (capabilities["variables"] == true);      
}

SieveSetVariable.prototype.require
    = function (imports)
{
  imports["variables"] = true;
}

SieveSetVariable.nodeName = function () {
  return "action/setvariable";
}

SieveSetVariable.nodeType  = function () {
  return "action";
}

SieveSetVariable.prototype.init
    = function (parser)
{  
  parser.extract("set");
  
  // ... eat the deadcode before the string...
  this.whiteSpace.init(parser);
      
  // the optional modifier
  
  // the name
  
  // the value
  //this.flaglist.init(parser)

  this.semicolon.init(parser);
    
  return this;
}


SieveSetVariable.prototype.toScript
    = function ()
{
  return "set"
    + this.whiteSpace.toScript()
    + this.flaglist.toScript()
    + this.semicolon.toScript();
}



/******************************************************************************/



/**
 *
 *    Usage:  string [MATCH-TYPE] [COMPARATOR]
           <source: string-list> <key-list: string-list>
 */ 
function SieveTestString(docshell,id)
{
  SieveAbstractElement.call(this,docshell,id); 

  this.whiteSpace = [];
  this.whiteSpace[0] = this._createByName("whitespace", " ");
  this.whiteSpace[1] = this._createByName("whitespace", " ");
  this.whiteSpace[2] = this._createByName("whitespace", " ");    

  this.matchType      = null;
  this.flaglist = this._createByName("stringlist");
}

SieveTestString.prototype = Object.create(SieveAbstractElement.prototype);
SieveTestString.prototype.constructor = SieveTestString;

SieveTestString.isElement
  = function(parser, lexer)
{ 
  return parser.startsWith("string");
}

SieveTestString.isCapable
    = function (capabilities)
{
  return (capabilities["variables"] == true);      
}

SieveTestString.prototype.require
    = function (imports)
{
  imports["variables"] = true;
  
  if (this.matchType)
    this.matchType.require(imports);
}

SieveTestString.nodeName = function () {
  return "test/string";
}

SieveTestString.nodeType  = function () {
  return "test";
}

SieveTestString.prototype.init
    = function (parser)
{
  parser.extract("hasflag")
  
  this.whiteSpace[0].init(parser)
  
  
  if (this._probeByName("match-type",parser))
  {
    this.matchType = this._createByName("match-type",parser);    
    this.whiteSpace[1].init(parser);
  }
  
  this.flaglist.init(parser);
  this.whiteSpace[2].init(parser);
      
  return this;
}    



SieveTestString.prototype.toScript
    = function ()
{
  return "hasflag"
    + this.whiteSpace[0].toScript()
    + ((this.matchType != null)?this.matchType[0].toScript():"")
    + ((this.matchType != null)?this.whiteSpace[1].toScript():"")
    + this.flaglist.toScript()
    + this.whiteSpace[2].toScript();
}




//*******************************************************************//

function SieveLowerFirstModifier(docshell, id) {
  SieveAbstractElement.call(this, docshell, id);
}

SieveLowerFirstModifier.prototype = Object.create(SieveAbstractElement.prototype);
SieveLowerFirstModifier.prototype.constructor = SieveLowerFirstModifier;

SieveLowerFirstModifier.nodeName = function () {
  return "modifier/lowerFirst";
}

SieveLowerFirstModifier.nodeType  = function () {
  return "modifier/";
}

SieveLowerFirstModifier.isElement
    = function (parser, lexer)
{    

  if (parser.startsWith(":lowerfirst"))
    return true;
  
  return false;
}


SieveLowerFirstModifier.prototype.init
    = function (parser)
{	
  parser.extract(":lowerfirst");    
  return this;
}

SieveLowerFirstModifier.prototype.toScript
    = function ()
{    
  return ":lowerfirst";
}


//*******************************************************************//

function SieveModifier(docshell, id) {
	// call super constructor.
	SieveAbstractElement.call(this, docshell, id);
	
	// the default matchtype is by definition a :is
	this.type = this._createByName("match-type/is",":is");
	this.optional = true;
}

SieveModifier.prototype = Object.create(SieveAbstractElement.prototype);
SieveModifier.prototype.constructor = SieveModifier;

SieveModifier.nodeName = function () {
  return "match-type";
}

SieveModifier.nodeType  = function () {
  return "comparison";
}

SieveModifier.isElement
    = function (parser, lexer)
{
  return lexer.probeByClass(["match-type/"], parser);
}

SieveModifier.prototype.require
    = function (imports)
{
  this.type.require(imports);
}

SieveModifier.prototype.init
    = function (parser)    
{
  this.type = this._createByClass(["match-type/"],parser);

  if (this.type instanceof SieveIsMatch)
    this.optional = false;  
  
  return this;
}

SieveModifier.prototype.modifier
    = function (value)
{
  if(typeof(value) === "undefined")
    return this.type.toScript();
    
  value = value.toLowerCase();
  
  if (!this._probeByClass(["match-type/"],value)) 
    throw "Unkonwn Match type >>"+value+"<<";
    
  this.type = this._createByClass(["match-type/"],value);	     
  
  return this;
}

SieveModifier.prototype.toScript
    = function ()
{
  if (this.isOptional())
    return "";
    
  return this.type.toScript();
}


/******************************************************************************/

if (!SieveLexer)
  throw "Could not register variables extension";

SieveLexer.register(SieveSetVariable);
            
SieveLexer.register(SieveTestString);

//SieveLexer.register(SieveLengthModifier);
//SieveLexer.register(SieveQuoteWildcardModifier);
//SieveLexer.register(SieveUpperModifier);
//SieveLexer.register(SieveLowerModifier);
//SieveLexer.register(SieveUpperFirstModifier);
//SieveLexer.register(SieveLowerFirstModifier);

SieveLexer.register(SieveModifier);
 